--- ORIGINALS/mountpoint.c	Sat May 19 12:14:04 2007
+++ ./mountpoint.c	Tue Jul 31 16:34:46 2007
@@ -19,6 +19,7 @@
 #elif defined(HAVE_SYS_MNTTAB_H)
 #  include <stdio.h>
 #  include <sys/mnttab.h> /* Solaris */
+#  include <sys/mntent.h>
 #else
 #  define MOUNTPOINT_UNKNOWN
 #endif
@@ -68,11 +69,15 @@
 #else
 	/* Linux, Solaris: /etc/mtab reading */
 #ifdef HAVE_SYS_MNTTAB_H
+	union {
 	struct mnttab ent;
+		struct extmnttab ext;
+	} ent;
 #else
 	struct mntent *ent;
+	struct stat st2;
 #endif
-	struct stat st, st2;
+	struct stat st;
 	const char *device_path = NULL, *mount_path = NULL, *type = NULL;
 	unsigned int block_size;
 	FILE *f;
@@ -94,16 +99,19 @@
 		i_error("fopen(%s) failed: %m", MTAB_PATH);
 		return -1;
 	}
-	while ((getmntent(f, &ent)) == 0) {
-		if (strcmp(ent.mnt_fstype, MNTTYPE_SWAP) == 0 ||
-		    strcmp(ent.mnt_fstype, MNTTYPE_IGNORE) == 0)
+	while ((getextmntent(f, &ent.ext, sizeof(ent.ext))) == 0) {
+		if (hasmntopt(&ent.ent, MNTOPT_IGNORE) != NULL)
 			continue;
 
-		if (stat(ent.mnt_mountp, &st2) == 0 &&
-		    CMP_DEV_T(st.st_dev, st2.st_dev)) {
-			device_path = ent.mnt_special;
-			mount_path = ent.mnt_mountp;
-			type = ent.mnt_fstype;
+		/* mnt_type contains tmpfs with swap */
+		if (strcmp(ent.ent.mnt_special, MNTTYPE_SWAP) == 0)
+			continue;
+
+		if (major(st.st_dev) == ent.ext.mnt_major &&
+		    minor(st.st_dev) == ent.ext.mnt_minor) {
+			device_path = ent.ent.mnt_special;
+			mount_path = ent.ent.mnt_mountp;
+			type = ent.ent.mnt_fstype;
 			break;
 		}
 	}
